@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2022	PgPool Global Development Group
+ * Copyright (c) 2003-2017	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -62,6 +62,7 @@
 #include "auth/pool_hba.h"
 #include "utils/pool_stream.h"
 
+char		remote_ps_data[NI_MAXHOST]; /* used for set_ps_display */
 static POOL_CONNECTION_POOL_SLOT * slot;
 static volatile sig_atomic_t reload_config_request = 0;
 static volatile sig_atomic_t restart_request = 0;
@@ -72,7 +73,15 @@ static RETSIGTYPE reload_config_handler(int sig);
 static void reload_config(void);
 static RETSIGTYPE health_check_timer_handler(int sig);
 
-static bool check_backend_down_request(int node, bool done_requests);
+#ifdef HEALTHCHECK_OPTS
+#if HEALTHCHECK_OPTS > 0
+#define HEALTHCHECK_DEBUG
+#endif
+#endif
+
+#ifdef HEALTHCHECK_DEBUG
+static bool check_backend_down_request(int node);
+#endif
 
 #undef CHECK_REQUEST
 #define CHECK_REQUEST \
@@ -97,11 +106,12 @@ static bool check_backend_down_request(int node, bool done_requests);
 /*
 * health check child main loop
 */
-void do_health_check_child(int *node_id)
+void
+do_health_check_child(int *node_id)
 {
-    sigjmp_buf	local_sigjmp_buf;
+	sigjmp_buf	local_sigjmp_buf;
 	MemoryContext HealthCheckMemoryContext;
-	char psbuffer[NI_MAXHOST];
+	char		psbuffer[NI_MAXHOST];
 
 	ereport(DEBUG1,
 			(errmsg("I am health check process pid:%d DB node id:%d", getpid(), *node_id)));
@@ -122,9 +132,9 @@ void do_health_check_child(int *node_id)
 	signal(SIGUSR2, SIG_IGN);
 	signal(SIGPIPE, SIG_IGN);
 
-    /* Create per loop iteration memory context */
+	/* Create per loop iteration memory context */
 	HealthCheckMemoryContext = AllocSetContextCreate(TopMemoryContext,
-                                             "health_check_main_loop",
+													 "health_check_main_loop",
 													 ALLOCSET_DEFAULT_MINSIZE,
 													 ALLOCSET_DEFAULT_INITSIZE,
 													 ALLOCSET_DEFAULT_MAXSIZE);
@@ -137,7 +147,7 @@ void do_health_check_child(int *node_id)
 	/* Initialize per process context */
 	pool_init_process_context();
 
-    if (sigsetjmp(local_sigjmp_buf, 1) != 0)
+	if (sigsetjmp(local_sigjmp_buf, 1) != 0)
 	{
 		pool_signal(SIGALRM, SIG_IGN);
 		error_context_stack = NULL;
@@ -151,7 +161,7 @@ void do_health_check_child(int *node_id)
 
 	for (;;)
 	{
-        MemoryContextSwitchTo(HealthCheckMemoryContext);
+		MemoryContextSwitchTo(HealthCheckMemoryContext);
 		MemoryContextResetAndDeleteChildren(HealthCheckMemoryContext);
 
 		CHECK_REQUEST;
@@ -167,12 +177,16 @@ void do_health_check_child(int *node_id)
 		 */
 		else if (pool_config->health_check_params[*node_id].health_check_period > 0)
 		{
-			bool result;
+			bool		result;
 			BackendInfo *bkinfo = pool_get_node_info(*node_id);
 
 			result = establish_persistent_connection(*node_id);
 
+#ifdef HEALTHCHECK_DEBUG
+			if (check_backend_down_request(*node_id) || (result && slot == NULL))
+#else
 			if (result && slot == NULL)
+#endif
 			{
 				if (POOL_DISALLOW_TO_FAILOVER(BACKEND_INFO(*node_id).flag))
 				{
@@ -182,7 +196,7 @@ void do_health_check_child(int *node_id)
 				}
 				else
 				{
-					bool partial;
+					bool		partial;
 
 					ereport(LOG, (errmsg("health check failed on node %d (timeout:%d)",
 										 *node_id, health_check_timer_expired)));
@@ -201,7 +215,7 @@ void do_health_check_child(int *node_id)
 					}
 				}
 			}
-			else if (slot && bkinfo->backend_status == CON_DOWN && bkinfo->quarantine == true)
+			else if (bkinfo->backend_status == CON_DOWN && bkinfo->quarantine == true)
 			{
 				/* The node has become reachable again. Reset
 				 * the quarantine state
@@ -221,11 +235,11 @@ void do_health_check_child(int *node_id)
  * Establish persistent connection to backend.
  * Return true if connection test is done.
  */
-static bool establish_persistent_connection(int node)
+static bool
+establish_persistent_connection(int node)
 {
 	BackendInfo *bkinfo;
-	int retry_cnt;
-	char		*dbname;
+	int			retry_cnt;
 
 	bkinfo = pool_get_node_info(node);
 
@@ -243,9 +257,7 @@ static bool establish_persistent_connection(int node)
 	 * If database is not specified, "postgres" database is assumed.
 	 */
 	if (*pool_config->health_check_params[node].health_check_database == '\0')
-		dbname = "postgres";
-	else
-		dbname = pool_config->health_check_params[node].health_check_database;
+		pool_config->health_check_params[node].health_check_database = "postgres";
 
 	/*
 	 * Try to connect to the database.
@@ -254,12 +266,14 @@ static bool establish_persistent_connection(int node)
 	{
 		retry_cnt = pool_config->health_check_params[node].health_check_max_retries;
 
+		char	   *password = get_pgpool_config_user_password(pool_config->health_check_params[node].health_check_user,
+															   pool_config->health_check_params[node].health_check_password);
+
 		do
 		{
 			/*
-			 * Set health checker timeout. we want to detect
-			 * communication path failure much earlier before
-			 * TCP/IP stack detects it.
+			 * Set health checker timeout. we want to detect communication
+			 * path failure much earlier before TCP/IP stack detects it.
 			 */
 			if (pool_config->health_check_params[node].health_check_timeout > 0)
 			{
@@ -272,9 +286,9 @@ static bool establish_persistent_connection(int node)
 
 			slot = make_persistent_db_connection_noerror(node, bkinfo->backend_hostname,
 														 bkinfo->backend_port,
-														 dbname,
+														 pool_config->health_check_params[node].health_check_database,
 														 pool_config->health_check_params[node].health_check_user,
-														 pool_config->health_check_params[node].health_check_password, false);
+														 password ? password : "", false);
 
 			if (pool_config->health_check_params[node].health_check_timeout > 0)
 			{
@@ -283,19 +297,6 @@ static bool establish_persistent_connection(int node)
 				CLEAR_ALARM;
 			}
 
-			/*
-			 * If health check test is enabled, check if fake down request is
-			 * set.  If set, discard_persistent_connection will set slot to
-			 * NULL.  This simulates a connection failure.
-			 */
-			if (pool_config->health_check_test)
-			{
-				if (slot && check_backend_down_request(node, false) == true)
-				{
-					discard_persistent_connection(node);
-				}
-			}
-
 			if (slot)
 			{
 				if (retry_cnt != pool_config->health_check_params[node].health_check_max_retries)
@@ -304,7 +305,7 @@ static bool establish_persistent_connection(int node)
 							(errmsg("health check retrying on DB node: %d succeeded",
 									node)));
 				}
-				break;	/* Success */
+				break;			/* Success */
 			}
 
 			retry_cnt--;
@@ -319,6 +320,9 @@ static bool establish_persistent_connection(int node)
 				sleep(pool_config->health_check_params[node].health_check_retry_delay);
 			}
 		} while (retry_cnt >= 0);
+
+		if (password)
+			pfree(password);
 	}
 	return true;
 }
@@ -326,7 +330,8 @@ static bool establish_persistent_connection(int node)
 /*
  * Discard persistent connection to backend
  */
-static void discard_persistent_connection(int node)
+static void
+discard_persistent_connection(int node)
 {
 	if (slot)
 	{
@@ -337,7 +342,7 @@ static void discard_persistent_connection(int node)
 
 static RETSIGTYPE my_signal_handler(int sig)
 {
-	int save_errno = errno;
+	int			save_errno = errno;
 
 	POOL_SETMASK(&BlockSig);
 
@@ -366,68 +371,72 @@ static RETSIGTYPE my_signal_handler(int sig)
 
 static RETSIGTYPE reload_config_handler(int sig)
 {
-	int save_errno = errno;
+	int			save_errno = errno;
+
 	POOL_SETMASK(&BlockSig);
 	reload_config_request = 1;
 	POOL_SETMASK(&UnBlockSig);
 	errno = save_errno;
 }
 
-static void reload_config(void)
+static void
+reload_config(void)
 {
 	ereport(LOG,
 			(errmsg("reloading config file")));
-    MemoryContext oldContext = MemoryContextSwitchTo(TopMemoryContext);
+	MemoryContext oldContext = MemoryContextSwitchTo(TopMemoryContext);
+
 	pool_get_config(get_config_file_name(), CFGCXT_RELOAD);
-    MemoryContextSwitchTo(oldContext);
+	MemoryContextSwitchTo(oldContext);
 	if (pool_config->enable_pool_hba)
 		load_hba(get_hba_file_name());
 	reload_config_request = 0;
 }
+
 /*
  * health check timer handler
  */
 static RETSIGTYPE health_check_timer_handler(int sig)
 {
-	int save_errno = errno;
+	int			save_errno = errno;
+
 	POOL_SETMASK(&BlockSig);
 	health_check_timer_expired = 1;
 	POOL_SETMASK(&UnBlockSig);
 	errno = save_errno;
 }
 
+#ifdef HEALTHCHECK_DEBUG
+
 /*
  * Node down request file. In the file, each line consists of "backend node
  * id", tab and "down".  If such a line found, check_backend_down_request()
  * will return true.
  */
 
-# define BACKEND_DOWN_REQUEST_FILE	"backend_down_request"
+#define BACKEND_DOWN_REQUEST_FILE	"backend_down_request"
 
 /*
  * Check backend down request file with specified backend node id.  If it's
  * down ("down"), returns true and set the status to "already_down" to
- * prevent repeating failover. If it's other than "down", returns false.
- *
- * When done_requests is true (second arg to function) the function returns
- * true if the node has already_done status in the file.
- */
+ * prevent repeatable * failover. If it's other than "down", returns false.
+*/
 
 static bool
-check_backend_down_request(int node, bool done_requests)
+check_backend_down_request(int node)
 {
-	static char	backend_down_request_file[POOLMAXPATHLEN];
-	FILE *fd;
-	int i;
+	static char backend_down_request_file[POOLMAXPATHLEN];
+	FILE	   *fd;
+	int			i;
 #define MAXLINE 128
-	char linebuf[MAXLINE];
-	char readbuf[MAXLINE];
-	char buf[MAXLINE];
-	char *writebuf;
-	char *p;
-	bool found = false;
-	int node_id;
-	char status[MAXLINE];
+	char		linebuf[MAXLINE];
+	char		readbuf[MAXLINE];
+	char		buf[MAXLINE];
+	char	   *writebuf;
+	char	   *p;
+	bool		found = false;
+	int			node_id;
+	char		status[MAXLINE];
 
 	if (backend_down_request_file[0] == '\0')
 	{
@@ -441,21 +450,21 @@ check_backend_down_request(int node, bool done_requests)
 		ereport(WARNING,
 				(errmsg("check_backend_down_request: failed to open file %s",
 						backend_down_request_file),
-				 errdetail("\"%s\"",strerror(errno))));
+				 errdetail("\"%s\"", strerror(errno))));
 		return false;
 	}
 
 	writebuf = NULL;
 
-	for (i=0;;i++)
+	for (i = 0;; i++)
 	{
-		readbuf[MAXLINE-1] = '\0';
-		if (fgets(readbuf, MAXLINE-1, fd) == 0)
+		readbuf[MAXLINE - 1] = '\0';
+		if (fgets(readbuf, MAXLINE - 1, fd) == 0)
 			break;
 
-		strncpy(buf, readbuf, sizeof(buf) - 1);
-		if (strlen(readbuf) > 0 && readbuf[strlen(readbuf)-1] == '\n')
-			buf[strlen(readbuf)-1] = '\0';
+		strncpy(buf, readbuf, sizeof(buf));
+		if (strlen(readbuf) > 0 && readbuf[strlen(readbuf) - 1] == '\n')
+			buf[strlen(readbuf) - 1] = '\0';
 
 		p = readbuf;
 		if (found == false)
@@ -471,11 +480,11 @@ check_backend_down_request(int node, bool done_requests)
 
 		if (writebuf == NULL)
 		{
-			writebuf = malloc(strlen(p)+1);
-			memset(writebuf, 0, strlen(p)+1);
+			writebuf = malloc(strlen(p) + 1);
+			memset(writebuf, 0, strlen(p) + 1);
 		}
 		else
-			writebuf = realloc(writebuf, strlen(p)+strlen(writebuf)+1);
+			writebuf = realloc(writebuf, strlen(p) + strlen(writebuf) + 1);
 		if (!writebuf)
 		{
 			fclose(fd);
@@ -486,11 +495,30 @@ check_backend_down_request(int node, bool done_requests)
 
 	fclose(fd);
 
-	if (writebuf)
-		free(writebuf);
-
 	if (!found)
 		return false;
 
+	fd = fopen(backend_down_request_file, "w");
+	if (!fd)
+	{
+		ereport(WARNING,
+				(errmsg("check_backend_down_request: failed to open file for writing %s",
+						backend_down_request_file),
+				 errdetail("\"%s\"", strerror(errno))));
+		return false;
+	}
+
+	if (fwrite(writebuf, 1, strlen(writebuf), fd) != strlen(writebuf))
+	{
+		ereport(WARNING,
+				(errmsg("check_backend_down_request: failed to write %s",
+						backend_down_request_file),
+				 errdetail("\"%s\"", strerror(errno))));
+		fclose(fd);
+		return false;
+	}
+	fclose(fd);
+
 	return true;
 }
+#endif
