@@ -16,6 +16,7 @@ MODE=install
 PG_INSTALL_DIR=/usr/local/pgsql/bin
 PGPOOL_PATH=/usr/local
 JDBC_DRIVER=/usr/local/pgsql/share/postgresql-9.2-1003.jdbc4.jar
+export USE_REPLICATION_SLOT=true
 export log=$dir/log
 fail=0
 ok=0
@@ -37,7 +38,7 @@ function install_pgpool
 
 	test -d $log || mkdir $log
         
-	make install WATCHDOG_DEBUG=1 -C $dir/../../ -e prefix=${PGPOOL_PATH} >& regression.log 2>&1
+	make install HEALTHCHECK_DEBUG=1 -C $dir/../../ -e prefix=${PGPOOL_PATH} >& regression.log 2>&1
 
 	if [ $? != 0 ];then
 	    echo "make install failed"
@@ -91,18 +92,13 @@ function export_env_vars
  	fi
 	
 	echo "using pgpool-II at "$PGPOOL_PATH
-
-	export PGPOOL_VERSION=`$PGPOOL_PATH/bin/pgpool --version 2>&1`
-
 	export PGPOOL_INSTALL_DIR=$PGPOOL_PATH
-	# where to look for pgpool.conf.sample files.
-	export PGPOOLDIR=${PGPOOLDIR:-"$PGPOOL_INSTALL_DIR/etc"}
 
 	PGPOOLLIB=${PGPOOL_INSTALL_DIR}/lib
 	if [ -z "$LD_LIBRARY_PATH" ];then
-	    export LD_LIBRARY_PATH=${PGPOOLLIB}:${PGLIB}
+	    export LD_LIBRARY_PATH=$PGPOOLLIB
 	else
-	    export LD_LIBRARY_PATH=${PGPOOLLIB}:${PGLIB}:${LD_LIBRARY_PATH}
+	    export LD_LIBRARY_PATH=${PGPOOLLIB}:${LD_LIBRARY_PATH}
 	fi
 
 	export TESTLIBS=$dir/libs.sh
@@ -110,16 +106,16 @@ function export_env_vars
 	export JDBC_DRIVER=$JDBC_DRIVER
 	export PGBENCH_PATH=$PGBENCH_PATH
 	export PGSOCKET_DIR=$PGSOCKET_DIR
-	export LANG=C
-	export ENABLE_TEST=true
+	export PGVERSION=`$PGBIN/initdb -V|awk '{print $3}'|sed 's/\..*//'`
 }
 function print_info
 {
 	echo ${CBLUE}"*************************"${CNORM}
+
 	echo "REGRESSION MODE          : "${CBLUE}$MODE${CNORM}
-	echo "Pgpool-II version        : "${CBLUE}$PGPOOL_VERSION${CNORM}
-	echo "Pgpool-II install path   : "${CBLUE}$PGPOOL_PATH${CNORM}
+	echo "PGPOOL-II                : "${CBLUE}$PGPOOL_PATH${CNORM}
 	echo "PostgreSQL bin           : "${CBLUE}$PGBIN${CNORM}
+	echo "PostgreSQL Major version : "${CBLUE}$PGVERSION${CNORM}
 	echo "pgbench                  : "${CBLUE}$PGBENCH_PATH${CNORM}
 	echo "PostgreSQL jdbc          : "${CBLUE}$JDBC_DRIVER${CNORM}
 	echo ${CBLUE}"*************************"${CNORM}
@@ -167,11 +163,11 @@ if [ "$MODE" = "install" ]; then
 
 elif [ "$MODE" = "noinstall" ]; then
 	echo not installing pgpool for the tests ...
-	if [[ -n "$PGPOOL_INSTALL_DIR" ]]; then
-		PGPOOL_PATH=$PGPOOL_INSTALL_DIR
+	if [[ -n "$PGPOOL_INSTALL_PATH" ]]; then
+		PGPOOL_PATH=$PGPOOL_INSTALL_PATH
 	fi
-        export PGPOOL_SETUP=$PGPOOL_PATH/bin/pgpool_setup
-        export WATCHDOG_SETUP=$PGPOOL_PATH/bin/watchdog_setup
+        export PGPOOL_SETUP=$dir/../pgpool_setup
+        export WATCHDOG_SETUP=$dir/../watchdog_setup
 else
 	echo $MODE : Invalid mode
 	exit -1
@@ -180,7 +176,6 @@ fi
 verify_pginstallation
 export_env_vars
 print_info
-source $TESTLIBS
 
 #Start executing tests
 rm -fr $log
@@ -198,16 +193,8 @@ for i in $dirs
 do
 	cd $i
 	echo -n "testing $i..."
-	clean_all
 	timeout $TIMEOUT ./test.sh > $log/$i 2>&1
 	rtn=$?
-
-	check_segfault
-	if [ $? -eq 0 ];then
-		echo "fail: Segmentation fault detected" >> $log/$i
-		rtn=1
-	fi
-
 	if [ $rtn = 0 ];then
 		echo ${CGREEN}"ok."${CNORM}
 		ok=`expr $ok + 1`
